@@ -23,11 +23,12 @@
 #include "config.h"
 #include <stdio.h>	/* FILE, tmpfile, fileno, fclose */
 #include <gio/gio.h>
-#define GETTEXT_PACKAGE PACKAGE
+#define GETTEXT_PACKAGE "libparlatype"
 #include <glib/gi18n-lib.h>
 #include <gst/gst.h>
 #include <gst/audio/audio.h>
 #include <sys/stat.h>	/* fstat */
+#include "pt-i18n.h"
 #include "pt-waveloader.h"
 
 struct _PtWaveloaderPrivate
@@ -90,7 +91,7 @@ on_wave_loader_new_pad (GstElement *bin,
 			gpointer    user_data)
 {
 	if (!gst_element_link (bin, GST_ELEMENT (user_data))) {
-		GST_WARNING ("Can't link output of wave decoder to converter.");
+		GST_WARNING ("Can’t link output of wave decoder to converter.");
 	}
 }
 
@@ -130,14 +131,14 @@ setup_pipeline (PtWaveloader *wl)
 	result = gst_element_link (src, dec);
 	if (!result) {
 		GST_WARNING_OBJECT (wl->priv->pipeline,
-			"Can't link wave loader pipeline (src ! dec ! conv ! fmt ! sink).");
+			"Can’t link wave loader pipeline (src ! dec ! conv ! fmt ! sink).");
 		return result;
 	}
 
 	result = gst_element_link_many (conv, wl->priv->fmt, sink, NULL);
 	if (!result) {
 		GST_WARNING_OBJECT (wl->priv->pipeline,
-			"Can't link wave loader pipeline (conf ! fmt ! sink).");
+			"Can’t link wave loader pipeline (conf ! fmt ! sink).");
 		return result;
 	}
 
@@ -151,7 +152,7 @@ static gboolean
 check_progress (GTask *task)
 {
 	/* This timeout runs parallel to message bus (bus_handler()).
-	   If it's removed, the message bus has to be removed, too, and also
+	   If it’s removed, the message bus has to be removed, too, and also
 	   the other way round. */
 
 	PtWaveloader *wl = g_task_get_source_object (task);
@@ -217,8 +218,6 @@ bus_handler (GstBus     *bus,
 	GTask *task = (GTask *) data;
 	PtWaveloader *wl = g_task_get_source_object (task);
 
-	g_debug ("Message: %s; sent by: %s", GST_MESSAGE_TYPE_NAME (msg), GST_MESSAGE_SRC_NAME (msg));
-
 	switch (GST_MESSAGE_TYPE (msg)) {
 	case GST_MESSAGE_ERROR: {
 		gchar    *debug;
@@ -226,8 +225,14 @@ bus_handler (GstBus     *bus,
 
 		remove_timeout (wl);
 		gst_message_parse_error (msg, &error, &debug);
-		g_debug ("ERROR from element %s: %s", GST_OBJECT_NAME (msg->src), error->message);
-		g_debug ("Debugging info: %s", (debug) ? debug : "none");
+
+		/* Error is returned. Log the message here at level DEBUG,
+		   as higher levels will abort tests. */
+		   
+		g_log_structured (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
+				  "MESSAGE", "Error from element %s: %s", GST_OBJECT_NAME (msg->src), error->message);
+		g_log_structured (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
+				  "MESSAGE", "Debugging info: %s", (debug) ? debug : "none");
 		g_free (debug);
 		wl->priv->bus_watch_id = 0;
 		g_task_return_error (task, error);
@@ -261,8 +266,9 @@ bus_handler (GstBus     *bus,
 			gst_object_unref (pad);
 		}
 
-		g_debug ("sample decoded: channels=%d, rate=%d, length=%" GST_TIME_FORMAT,
-			wl->priv->channels, wl->priv->rate, GST_TIME_ARGS (wl->priv->duration));
+		g_log_structured (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
+				  "MESSAGE", "sample decoded: channels=%d, rate=%d, length=%" GST_TIME_FORMAT,
+		                  wl->priv->channels, wl->priv->rate, GST_TIME_ARGS (wl->priv->duration));
 
 		remove_timeout (wl);
 		wl->priv->bus_watch_id = 0;
@@ -283,7 +289,7 @@ bus_handler (GstBus     *bus,
  * pt_waveloader_load_finish:
  * @wl: a #PtWaveloader
  * @result: the #GAsyncResult passed to your #GAsyncReadyCallback
- * @error: (allow-none): a pointer to a NULL #GError, or NULL
+ * @error: (nullable): a pointer to a NULL #GError, or NULL
  *
  * Gives the result of the async load operation. A cancelled operation results
  * in an error, too.
@@ -303,13 +309,13 @@ pt_waveloader_load_finish (PtWaveloader  *wl,
 /**
  * pt_waveloader_load_async:
  * @wl: a #PtWaveloader
- * @cancellable: (allow-none): a #GCancellable or NULL
+ * @cancellable: (nullable): a #GCancellable or NULL
  * @callback: (scope async): a #GAsyncReadyCallback to call when the operation is complete
  * @user_data: (closure): user_data for callback
  *
  * Writes the raw sample data to a temporary file and also gets the number of
  * channels, the bit rate and the exact duration. Load only once and on success
- * the data can be retrieved as a #PtWavedata. It's a programmer's error trying
+ * the data can be retrieved as a #PtWavedata. It’s a programmer’s error trying
  * to retrieve the data without prior loading.
  *
  * Emits a progress signal while saving the temporary file.
@@ -398,10 +404,10 @@ pt_waveloader_get_duration (PtWaveloader *wl)
  * Returns wave data needed for visual representation as raw data. The
  * requested resolution is given as pixel per seconds, e.g. 100 means one second
  * is represented by 100 samples, is 100 pixels wide. The returned resolution
- * doesn't have to be necessarily exactly the requested resolution, it might be
+ * doesn’t have to be necessarily exactly the requested resolution, it might be
  * a bit differnt, depending on the bit rate.
  *
- * Return value: (transfer full): the #PtWavedata, after use free with pt_wavedata_free()
+ * Return value: (transfer full) (nullable): the #PtWavedata, after use free with pt_wavedata_free()
  */
 PtWavedata*
 pt_waveloader_get_data (PtWaveloader *wl,
@@ -418,13 +424,13 @@ pt_waveloader_get_data (PtWaveloader *wl,
 	/* stat temp file, query size in bytes and compute number of samples */
 	struct stat buf;
 
-	g_debug ("breakpoint 1");
 	if (fstat (wl->priv->fd, &buf) != 0) {
-		g_debug (_("Failed to open temporary file."));
+		g_log_structured (G_LOG_DOMAIN, G_LOG_LEVEL_WARNING,
+			          "MESSAGE", "Failed to open temporary file");
 		return NULL;
 	}
 
-	/* Adjust pixel per second ratio if there's a remainder */
+	/* Adjust pixel per second ratio if there’s a remainder */
 	for (i = pps; i > 10; i--) {
 		if (wl->priv->rate % i == 0) {
 			pps = i;
@@ -436,25 +442,32 @@ pt_waveloader_get_data (PtWaveloader *wl,
 	chunk_bytes = 2 * chunk_size;
 	wl->priv->data_size = buf.st_size / chunk_size;
 
-	/* Data size should match exactly duration or less, but sometimes it doesn't ... */
+	/* Data size should match exactly duration or less, but sometimes it doesn’t … */
 	while (calculate_duration (wl->priv->data_size, pps) > GST_TIME_AS_MSECONDS (wl->priv->duration)) {
-		g_debug ("adjusting array size");
+		g_log_structured (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
+			          "MESSAGE", "Adjusting array size");
 		wl->priv->data_size -= 2 * wl->priv->channels;
 	}
 
-	g_debug ("array size: %" G_GINT64_FORMAT " ", wl->priv->data_size);
-	g_debug ("samples: %" G_GINT64_FORMAT " ", wl->priv->data_size / 2 * wl->priv->channels);
-	g_debug ("pixels per sec: %d", pps);
+	g_log_structured (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
+		          "MESSAGE", "Array size: %" G_GINT64_FORMAT " ", wl->priv->data_size);
+	g_log_structured (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
+		          "MESSAGE", "samples: %" G_GINT64_FORMAT " ", wl->priv->data_size / 2 * wl->priv->channels);
+	g_log_structured (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
+		          "MESSAGE", "pixels per sec: %d", pps);
 
 	gint16 temp[chunk_size];
 
 	if (!(array = g_try_malloc (sizeof (gfloat) * wl->priv->data_size))) {
-		g_debug	("sample is too long or empty");
+		g_log_structured (G_LOG_DOMAIN, G_LOG_LEVEL_WARNING,
+			          "MESSAGE", "Sample is too long or empty");
 		return NULL;
 	}
 
 	if (lseek (wl->priv->fd, 0, SEEK_SET) != 0) {
-		g_debug ("sample not loaded!!!");
+		g_free (array);
+		g_log_structured (G_LOG_DOMAIN, G_LOG_LEVEL_WARNING,
+			          "MESSAGE", "Sample not loaded");
 		return NULL;
 	}
 
@@ -483,7 +496,6 @@ pt_waveloader_get_data (PtWaveloader *wl,
 					    pps);
 
 	g_free (array);
-	g_debug ("waveloader get data finished");
 	return data;
 }
 
@@ -498,7 +510,9 @@ pt_waveloader_init (PtWaveloader *wl)
 	GError *gst_error = NULL;
 	gst_init_check (NULL, NULL, &gst_error);
 	if (gst_error) {
-		g_debug ("PtWaveloader failed to init GStreamer");
+		g_log_structured (G_LOG_DOMAIN, G_LOG_LEVEL_ERROR,
+			          "MESSAGE", "PtWaveloader failed to init GStreamer: %s", gst_error->message);
+		g_clear_error (&gst_error);
 	}
 
 	wl->priv->pipeline = NULL;
@@ -631,7 +645,7 @@ pt_waveloader_class_init (PtWaveloaderClass *klass)
 
 /**
  * pt_waveloader_new:
- * @uri: (allow-none): URI of the audio file to load
+ * @uri: (nullable): URI of the audio file to load
  *
  * Returns a new #PtWaveloader. The @uri is not checked on construction, but
  * calling pt_waveloader_load_async() will fail with an error message.
@@ -643,6 +657,7 @@ pt_waveloader_class_init (PtWaveloaderClass *klass)
 PtWaveloader *
 pt_waveloader_new (gchar *uri)
 {
+	pt_i18n_init ();
 	return g_object_new (PT_TYPE_WAVELOADER,
 			     "uri", uri,
 			     NULL);
