@@ -1,4 +1,4 @@
-/* Copyright (C) Gabor Karsay 2016 <gabor.karsay@gmx.at>
+/* Copyright (C) Gabor Karsay 2016–2019 <gabor.karsay@gmx.at>
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as published
@@ -16,129 +16,90 @@
 
 
 #include "config.h"
-#include <stdlib.h>		/* exit() */
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>	
-#include "pt-app.h"
-#include "pt-window.h"
 #include "pt-preferences.h"
+#include "pt-window.h"
+#include "pt-app.h"
 
-struct _PtApp
+struct _PtAppPrivate
 {
-	GtkApplication parent;
+	gboolean       asr;
+	gboolean       atspi;
+	PtAsrSettings *asr_settings;
 };
 
-struct _PtAppClass
-{
-	GtkApplicationClass parent_class;
-};
 
-G_DEFINE_TYPE (PtApp, pt_app, GTK_TYPE_APPLICATION);
+G_DEFINE_TYPE_WITH_PRIVATE (PtApp, pt_app, GTK_TYPE_APPLICATION);
 
-static gboolean G_GNUC_NORETURN
-option_version_cb (const gchar *option_name,
-                   const gchar *value,
-                   gpointer     data,
-                   GError     **error)
-{
-	g_print ("%s %s\n", PACKAGE, VERSION);
-	exit (0);
-}
 
 static GOptionEntry options[] =
 {
 	{ "version",
 	  'v',
-	  G_OPTION_FLAG_NO_ARG,
-	  G_OPTION_ARG_CALLBACK,
-	  option_version_cb,
-	  N_("Show the application's version"),
+	  G_OPTION_FLAG_NONE,
+	  G_OPTION_ARG_NONE,
+	  NULL,
+	  N_("Show the application’s version"),
 	  NULL
 	},
+#ifdef HAVE_ASR
+	{ "with-asr",
+	  'a',
+	  G_OPTION_FLAG_NONE,
+	  G_OPTION_ARG_NONE,
+	  NULL,
+	  N_("Enable automatic speech recognition"),
+	  NULL
+	},
+	{ "textpad",
+	  't',
+	  G_OPTION_FLAG_NONE,
+	  G_OPTION_ARG_NONE,
+	  NULL,
+	  N_("Use internal textpad for automatic speech recognition"),
+	  NULL
+	},
+#endif
 	{ NULL }
 };
 
-static void
-recent_cb (GSimpleAction *action,
-	   GVariant      *parameter,
-	   gpointer       app)
-{
-	GtkWidget	 *dialog;
-	GtkWindow	 *win;
-	GtkRecentInfo	 *info = NULL;
-	GtkRecentFilter  *filter;
-	GtkRecentFilter  *filter_all;
-	const gchar	 *uri = NULL;
-
-	win = gtk_application_get_active_window (app);
-	dialog = gtk_recent_chooser_dialog_new (
-			_("Recent Files"),
-			win,
-			_("_Cancel"), GTK_RESPONSE_CANCEL,
-			_("_Open"), GTK_RESPONSE_ACCEPT,
-			NULL);
-
-	gtk_recent_chooser_set_sort_type (GTK_RECENT_CHOOSER (dialog), GTK_RECENT_SORT_MRU); /* Most Recently Used first */
-	gtk_recent_chooser_set_show_icons (GTK_RECENT_CHOOSER (dialog), TRUE);
-	filter = gtk_recent_filter_new ();
-	gtk_recent_filter_set_name (filter, _("Parlatype"));
-	gtk_recent_filter_add_application (filter, "parlatype");
-	gtk_recent_filter_add_mime_type (filter, "audio/*");
-	gtk_recent_chooser_add_filter (GTK_RECENT_CHOOSER (dialog), filter);
-
-	filter_all = gtk_recent_filter_new ();
-	gtk_recent_filter_set_name (filter_all, _("All files"));
-	gtk_recent_filter_add_pattern (filter_all, "*");
-	gtk_recent_chooser_add_filter (GTK_RECENT_CHOOSER (dialog), filter_all);
-
-	gtk_recent_chooser_set_filter (GTK_RECENT_CHOOSER (dialog), filter);
-
-	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
-		info = gtk_recent_chooser_get_current_item (GTK_RECENT_CHOOSER (dialog));
-		if (info)
-			uri = gtk_recent_info_get_uri (info);
-	}
-
-	gtk_widget_destroy (dialog);
-
-	if (uri) {
-		/* GStreamer has problems with uris from recent chooser,
-		   as a workaround use GFile magic */
-		GFile *file = g_file_new_for_uri (uri);
-		gchar *tmp = g_file_get_uri (file);
-		pt_window_open_file (PT_WINDOW (win), tmp);
-		g_free (tmp);
-		g_object_unref (file);
-		gtk_recent_info_unref (info);
-	}
-}
-
 static void
 open_cb (GSimpleAction *action,
 	 GVariant      *parameter,
 	 gpointer       app)
 {
-	GtkWidget     *dialog;
+	GtkFileChooserNative *dialog;
 	GtkWindow     *win;
 	const char    *home_path;
-	GFile	      *home;
+	gchar         *current_uri = NULL;
+	GFile	      *current, *dir;
 	GtkFileFilter *filter_audio;
 	GtkFileFilter *filter_all;
 	gchar         *uri = NULL;
 
 	win = gtk_application_get_active_window (app);
-	dialog = gtk_file_chooser_dialog_new (
+	dialog = gtk_file_chooser_native_new (
 			_("Open Audio File"),
 			win,
 			GTK_FILE_CHOOSER_ACTION_OPEN,
-			_("_Cancel"), GTK_RESPONSE_CANCEL,
-			_("_Open"), GTK_RESPONSE_ACCEPT,
-			NULL);
+			_("_Open"),
+			_("_Cancel"));
 
-	/* Set current folder to user's home directory */
-	home_path = g_get_home_dir ();
-	home = g_file_new_for_path (home_path);
-	gtk_file_chooser_set_current_folder_file (GTK_FILE_CHOOSER (dialog), home, NULL);
+	/* Set current folder to the folder with the currently open file
+	   or to user’s home directory */
+	current_uri = pt_window_get_uri (PT_WINDOW (win));
+	if (current_uri) {
+		current = g_file_new_for_uri (current_uri);
+		dir = g_file_get_parent (current);
+		g_free (current_uri);
+		g_object_unref (current);
+	} else {
+		home_path = g_get_home_dir ();
+		dir = g_file_new_for_path (home_path);
+	}
+	gtk_file_chooser_set_current_folder_file (
+		GTK_FILE_CHOOSER (dialog), dir, NULL);
 
 	filter_audio = gtk_file_filter_new ();
 	filter_all = gtk_file_filter_new ();
@@ -150,12 +111,12 @@ open_cb (GSimpleAction *action,
 	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter_all);
 	gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (dialog), filter_audio);
 
-	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
+	if (gtk_native_dialog_run (GTK_NATIVE_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
 		uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (dialog));
 	}
 
-	g_object_unref (home);
-	gtk_widget_destroy (dialog);
+	g_object_unref (dir);
+	g_object_unref (dialog);
 
 	if (uri) {
 		pt_window_open_file (PT_WINDOW (win), uri);
@@ -184,23 +145,16 @@ help_cb (GSimpleAction *action,
 	GError *error = NULL;
 	gchar  *errmsg;
 
-#if GTK_CHECK_VERSION(3,22,0)
 	gtk_show_uri_on_window (
 			win,
 			"help:com.github.gkarsay.parlatype",
 			GDK_CURRENT_TIME,
 			&error);
-#else
-	gtk_show_uri (gtk_widget_get_screen (GTK_WIDGET (win)),
-	              "help:com.github.gkarsay.parlatype",
-	              GDK_CURRENT_TIME,
-	              &error);
-#endif
 
 	if (error) {
 		/* Translators: %s is a detailed error message */
 		errmsg = g_strdup_printf (_("Error opening help: %s"), error->message);
-		pt_error_message (PT_WINDOW (win), errmsg);
+		pt_error_message (PT_WINDOW (win), errmsg, NULL);
 		g_free (errmsg);
 		g_error_free (error);
 	}
@@ -234,7 +188,7 @@ about_cb (GSimpleAction *action,
 			gtk_application_get_active_window (app),
 			"program_name", _("Parlatype"),
 			"version", PACKAGE_VERSION,
-			"copyright", "© Gabor Karsay 2016–2018",
+			"copyright", "© Gabor Karsay 2016–2019",
 			"comments", _("A basic transcription utility"),
 			"logo-icon-name", "com.github.gkarsay.parlatype",
 			"authors", authors,
@@ -255,36 +209,48 @@ quit_cb (GSimpleAction *action,
 
 const GActionEntry app_actions[] = {
 	{ "open", open_cb, NULL, NULL, NULL },
-	{ "recent", recent_cb, NULL, NULL, NULL },
 	{ "prefs", prefs_cb, NULL, NULL, NULL },
 	{ "help", help_cb, NULL, NULL, NULL },
 	{ "about", about_cb, NULL, NULL, NULL },
 	{ "quit", quit_cb, NULL, NULL, NULL }
 };
 
+PtAsrSettings*
+pt_app_get_asr_settings (PtApp *app)
+{
+	g_assert (PT_IS_APP (app));
+	return app->priv->asr_settings;
+}
+
+gboolean
+pt_app_get_atspi (PtApp *app)
+{
+	g_assert (PT_IS_APP (app));
+	return app->priv->atspi;
+}
+
+gboolean
+pt_app_get_asr (PtApp *app)
+{
+	g_assert (PT_IS_APP (app));
+	return app->priv->asr;
+}
+
 static void
 pt_app_startup (GApplication *app)
 {
-	GtkBuilder *builder;
-	GMenuModel *app_menu;
-	GMenuModel *menubar;
-
-#if GLIB_CHECK_VERSION(2,42,0)
 	g_application_set_resource_base_path (app, "/com/github/gkarsay/parlatype");
-#endif
 	G_APPLICATION_CLASS (pt_app_parent_class)->startup (app);
 
 	g_action_map_add_action_entries (G_ACTION_MAP (app),
 	                                 app_actions, G_N_ELEMENTS (app_actions),
 	                                 app);
 
-#if GTK_CHECK_VERSION(3,12,0)
 	const gchar *quit_accels[2] = { "<Primary>Q", NULL };
 	const gchar *open_accels[2] = { "<Primary>O", NULL };
 	const gchar *copy_accels[2] = { "<Primary>C", NULL };
 	const gchar *insert_accels[2] = { "<Primary>V", NULL };
 	const gchar *goto_accels[2] = { "<Primary>G", NULL };
-	const gchar *recent_accels[2] = { "<Primary>R", NULL };
 	const gchar *help_accels[2] = { "F1", NULL };
 	const gchar *zoom_in_accels[3] = { "<Primary>plus", "<Primary>KP_Add", NULL };
 	const gchar *zoom_out_accels[3] = { "<Primary>minus", "<Primary>KP_Subtract", NULL };
@@ -295,9 +261,6 @@ pt_app_startup (GApplication *app)
 	gtk_application_set_accels_for_action (GTK_APPLICATION (app),
 			"app.open", open_accels);
 
-	gtk_application_set_accels_for_action (GTK_APPLICATION (app),
-			"app.recent", recent_accels);
-
 	gtk_application_set_accels_for_action (GTK_APPLICATION (app),
 			"win.copy", copy_accels);
 
@@ -315,82 +278,16 @@ pt_app_startup (GApplication *app)
 
 	gtk_application_set_accels_for_action (GTK_APPLICATION (app),
 			"win.zoom-out", zoom_out_accels);
-#else
-	gtk_application_add_accelerator (GTK_APPLICATION (app),
-			"<Primary>Q", "app.quit", NULL);
-
-	gtk_application_add_accelerator (GTK_APPLICATION (app),
-			"<Primary>O", "app.open", NULL);
-
-	gtk_application_add_accelerator (GTK_APPLICATION (app),
-			"<Primary>C", "win.copy", NULL);
-
-	gtk_application_add_accelerator (GTK_APPLICATION (app),
-			"<Primary>V", "win.insert", NULL);
-
-	gtk_application_add_accelerator (GTK_APPLICATION (app),
-			"<Primary>G", "win.goto", NULL);
-
-	gtk_application_add_accelerator (GTK_APPLICATION (app),
-			"<Primary>R", "app.recent", NULL);
-
-	gtk_application_add_accelerator (GTK_APPLICATION (app),
-			"F1", "app.help", NULL);
-
-	gtk_application_add_accelerator (GTK_APPLICATION (app),
-			"<Primary>plus", "win.zoom-in", NULL);
-	gtk_application_add_accelerator (GTK_APPLICATION (app),
-			"<Primary>KP_Add", "win.zoom-in", NULL);
-
-	gtk_application_add_accelerator (GTK_APPLICATION (app),
-			"<Primary>minus", "win.zoom-out", NULL);
-	gtk_application_add_accelerator (GTK_APPLICATION (app),
-			"<Primary>KP_Subtract", "win.zoom-out", NULL);
-#endif
-
-	builder = gtk_builder_new_from_resource ("/com/github/gkarsay/parlatype/menus.ui");
-
-#if GTK_CHECK_VERSION(3,14,0)
-	if (gtk_application_prefers_app_menu (GTK_APPLICATION (app))) {
-#else
-	gboolean show_menubar;
-
-	g_object_get (gtk_settings_get_default(),
-		      "gtk-shell-shows-menubar", &show_menubar,
-		      NULL);
-
-	if (show_menubar) {
-#endif
-#if GTK_CHECK_VERSION(3,20,0)
-		app_menu = G_MENU_MODEL (gtk_builder_get_object (builder, "appmenu"));
-#else
-		app_menu = G_MENU_MODEL (gtk_builder_get_object (builder, "appmenu-pre-3-20"));
-#endif
-		gtk_application_set_app_menu (GTK_APPLICATION (app), app_menu);
-	} else {
-#if GTK_CHECK_VERSION(3,20,0)
-		menubar = G_MENU_MODEL (gtk_builder_get_object (builder, "alternative-menu"));
-#else
-		menubar = G_MENU_MODEL (gtk_builder_get_object (builder, "alternative-menu-pre-3-20"));
-#endif
-		gtk_application_set_menubar (GTK_APPLICATION (app), menubar);
-	}
-
-	g_object_unref (builder);
 
 	/* Load custom css */
 	GtkCssProvider *provider;
-	GFile          *file;
-	file = g_file_new_for_uri ("resource:///com/github/gkarsay/parlatype/parlatype.css");
 	provider = gtk_css_provider_new ();
-	/* Note: since 3.16 there's also gtk_css_provider_load_from_resource () */
-	gtk_css_provider_load_from_file (provider, file, NULL);
+	gtk_css_provider_load_from_resource (provider, "/com/github/gkarsay/parlatype/parlatype.css");
 	gtk_style_context_add_provider_for_screen (
 			gdk_screen_get_default (),
 			GTK_STYLE_PROVIDER (provider),
 			GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
 	g_object_unref (provider);
-	g_object_unref (file);
 }
 
 static void
@@ -429,18 +326,99 @@ pt_app_open (GApplication  *app,
 	g_free (uri);
 }
 
-static void
-pt_app_class_init (PtAppClass *class)
+static gint
+pt_app_handle_local_options (GApplication *application,
+                             GVariantDict *options)
 {
-	G_APPLICATION_CLASS (class)->open = pt_app_open;
-	G_APPLICATION_CLASS (class)->activate = pt_app_activate;
-	G_APPLICATION_CLASS (class)->startup = pt_app_startup;
+	PtApp *app = PT_APP (application);
+
+	if (g_variant_dict_contains (options, "version")) {
+		g_print ("%s %s\n", PACKAGE, VERSION);
+		return 0;
+	}
+
+	if (g_variant_dict_contains (options, "with-asr")) {
+		app->priv->asr = TRUE;
+	}
+
+	if (g_variant_dict_contains (options, "textpad")) {
+		app->priv->atspi = FALSE;
+	}
+
+	return -1;
+}
+
+static gchar*
+get_asr_settings_filename (void)
+{
+	const gchar *userdir;
+	gchar	    *configdir;
+	GFile	    *create_dir;
+	gchar       *filename;
+
+	userdir = g_get_user_data_dir ();
+	configdir = g_build_path ("/", userdir, PACKAGE, NULL);
+	create_dir = g_file_new_for_path (configdir);
+	g_file_make_directory (create_dir, NULL, NULL);
+	filename = g_build_filename (configdir, "asr.ini", NULL);
+
+	g_free (configdir);
+	g_object_unref (create_dir);
+
+	return filename;
+}
+
+static gboolean
+is_flatpak (void)
+{
+	/* Flatpak sandboxes have a file ".flatpak-info" in their filesystem root.
+	   It's a keyfile, an example how to read is get_flatpak_information() in
+	   https://gitlab.gnome.org/GNOME/recipes/blob/master/src/gr-about-dialog.c */
+
+	return (g_file_test ("/.flatpak-info", G_FILE_TEST_EXISTS));
 }
 
 static void
 pt_app_init (PtApp *app)
 {
+	app->priv = pt_app_get_instance_private (app);
+
 	g_application_add_main_option_entries (G_APPLICATION (app), options);
+
+	/* In Flatpak's sandbox ATSPI doesn't work */
+	if (is_flatpak ())
+		app->priv->atspi = FALSE;
+	else
+		app->priv->atspi = TRUE;
+
+	app->priv->asr = FALSE;
+	gchar *asr_settings_filename;
+	asr_settings_filename = get_asr_settings_filename ();
+	app->priv->asr_settings = pt_asr_settings_new (asr_settings_filename);
+	g_free (asr_settings_filename);
+}
+
+static void
+pt_app_finalize (GObject *object)
+{
+	PtApp *app = PT_APP (object);
+
+	g_clear_object (&app->priv->asr_settings);
+
+	G_OBJECT_CLASS (pt_app_parent_class)->dispose (object);
+}
+
+static void
+pt_app_class_init (PtAppClass *klass)
+{
+	GApplicationClass    *gapp_class    = G_APPLICATION_CLASS (klass);
+	GObjectClass         *gobject_class = G_OBJECT_CLASS (klass);
+
+	gobject_class->finalize          = pt_app_finalize;
+	gapp_class->open                 = pt_app_open;
+	gapp_class->activate             = pt_app_activate;
+	gapp_class->startup              = pt_app_startup;
+	gapp_class->handle_local_options = pt_app_handle_local_options;
 }
 
 PtApp *
