@@ -109,7 +109,8 @@ static gboolean
 handle_dnd_data (PtWindow *win,
 		 gchar    *data)
 {
-	g_debug ("Received drag and drop: '%s'", data);
+	g_log_structured (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
+	                  "MESSAGE", "Received drag and drop: '%s'", data);
 
 	if (pt_player_goto_timestamp (win->priv->player, data))
 		return TRUE;
@@ -143,7 +144,8 @@ win_dnd_drop_cb (GtkWidget	*widget,
 	/* Choose the best target type */
 	iterate = gdk_drag_context_list_targets (context);
 	while (iterate != NULL) {
-		g_debug ("atom string: %s", gdk_atom_name (iterate->data));
+		g_log_structured (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
+			          "MESSAGE", "atom string: %s", gdk_atom_name (iterate->data));
 		if (iterate->data == gdk_atom_intern ("STRING", TRUE)) {
 			target_type = gdk_atom_intern ("STRING", TRUE);
 			have_target = TRUE;
@@ -160,7 +162,7 @@ win_dnd_drop_cb (GtkWidget	*widget,
 	if (have_target) {
 
                 /* Request the data from the source. */
-                gtk_drag_get_data (widget,	/* will receive 'drag-data-received' signal */
+                gtk_drag_get_data (widget,	/* will receive “drag-data-received” signal */
 				   context,	/* represents the current state of the DnD */
 				   target_type,
 				   time);
@@ -231,32 +233,6 @@ win_dnd_motion_cb (GtkWidget	  *widget,
 	return FALSE;
 }
 
-static void
-label_dnd_get_cb (GtkWidget	   *widget,
-		  GdkDragContext   *context,
-		  GtkSelectionData *data,
-		  guint	            info,
-		  guint	            time,
-		  gpointer	    user_data)
-{
-	PtWindow *win = user_data;
-	gchar *timestamp = NULL;
-
-	/* Note: timestamp when drag ended, maybe it would be better to produce
-	   the timestamp when drag starts. */
-	timestamp = pt_player_get_timestamp (win->priv->player);
-
-	if (!timestamp || (info != TARGET_STRING && info != TARGET_UTF8_STRING))
-		return;
-
-	gtk_selection_data_set (data,
-				gtk_selection_data_get_target (data),
-				8,	/* byte */
-				(guchar*) timestamp,
-				strlen (timestamp));
-	g_free (timestamp);
-}
-
 void
 pt_window_setup_dnd (PtWindow *win)
 {
@@ -282,17 +258,4 @@ pt_window_setup_dnd (PtWindow *win)
 				"drag_drop",
 				G_CALLBACK (win_dnd_drop_cb),
 				win);
-
-	/* The time label (its event box) is a source for timestamp drags */
-
-	gtk_drag_source_set (win->priv->label_box,
-				GDK_BUTTON1_MASK,
-				drag_target_string,
-				G_N_ELEMENTS (drag_target_string),
-				GDK_ACTION_COPY);
-
-	g_signal_connect (win->priv->label_box,
-				"drag_data_get",
-				G_CALLBACK (label_dnd_get_cb),
-				win);
 }
