@@ -19,12 +19,15 @@
 
 #include "config.h"
 #include <gtk/gtk.h>
+#include <pt-asr-output.h>
 #include <pt-player.h>
 #include <pt-wavedata.h>
 
 struct _PtWindowPrivate
 {
 	GSettings	 *editor;
+	PtAsrSettings    *asr_settings;
+	PtAsrOutput      *output;
 	GDBusProxy	 *proxy;
 	gint              dbus_watch_id;
 	GtkRecentManager *recent;
@@ -34,30 +37,35 @@ struct _PtWindowPrivate
 	GtkWidget  *progress_dlg;
 	gint	    progress_handler_id;
 
+	guint       output_handler_id1;
+	guint       output_handler_id2;
+
+	GtkClipboard *clip;
+	gulong        clip_handler_id;
+
+	/* Headerbar widgets */
 	GtkWidget  *button_open;
+	GtkWidget  *primary_menu_button;
+
+	/* Main window widgets */
 	GtkWidget  *button_play;
 	GtkWidget  *button_fast_back;	  // not used
 	GtkWidget  *button_fast_forward;  // not used
 	GtkWidget  *button_jump_back;
 	GtkWidget  *button_jump_forward;
 	GtkWidget  *volumebutton;
+	GtkWidget  *pos_menu_button;
 	GtkWidget  *pos_label;
-	GtkWidget  *label_box;
-	GtkWidget  *goto_button;
+	GMenuItem  *go_to_timestamp;
 	GtkWidget  *speed_scale;
 	GtkWidget  *waveviewer;
+	GMenuModel *secondary_menu;
 
 	PtWavedata *wavedata;
 
 	gint	    timer;
 	gdouble	    speed;
 
-	gint	    pause;
-	gint	    back;
-	gint	    forward;
-
-	gboolean    playing_selection;
-
 	guint	    owner_id;
 };
 
