@@ -1,4 +1,4 @@
-/* Copyright (C) Gabor Karsay 2016 <gabor.karsay@gmx.at>
+/* Copyright (C) Gabor Karsay 2016-2019 <gabor.karsay@gmx.at>
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as published
@@ -16,6 +16,7 @@
 
 
 #include "config.h"
+#include <stdlib.h>		/* exit() */
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 #ifdef GDK_WINDOWING_X11
@@ -25,6 +26,7 @@
 #include <pt-progress-dialog.h>
 #include <pt-waveviewer.h>
 #include "pt-app.h"
+#include "pt-asr-assistant.h"
 #include "pt-dbus-service.h"
 #include "pt-goto-dialog.h"
 #include "pt-mediakeys.h"
@@ -33,23 +35,16 @@
 #include "pt-window-private.h"
 
 
-enum
-{
-	PROP_0,
-	PROP_PAUSE,
-	PROP_BACK,
-	PROP_FORWARD,
-	N_PROPERTIES
-};
-
-static GParamSpec *obj_properties[N_PROPERTIES] = { NULL, };
-
 G_DEFINE_TYPE_WITH_PRIVATE (PtWindow, pt_window, GTK_TYPE_APPLICATION_WINDOW)
 
+void play_button_toggled_cb (GtkToggleButton *button, PtWindow *win);
+void jump_back_button_clicked_cb (GtkButton *button, PtWindow *win);
+void jump_forward_button_clicked_cb (GtkButton *button, PtWindow *win);
 
 void
 pt_error_message (PtWindow    *parent,
-		  const gchar *message)
+		  const gchar *message,
+		  const gchar *secondary_message)
 {
         GtkWidget *dialog;
 
@@ -60,6 +55,11 @@ pt_error_message (PtWindow    *parent,
 			GTK_BUTTONS_OK,
 			"%s", message);
 
+	if (secondary_message)
+		gtk_message_dialog_format_secondary_text (
+				GTK_MESSAGE_DIALOG (dialog),
+		                "%s", secondary_message);
+
         gtk_dialog_run (GTK_DIALOG (dialog));
         gtk_widget_destroy (dialog);
 }
@@ -71,16 +71,11 @@ copy_timestamp (GSimpleAction *action,
 {
 	PtWindow *win;
 	win = PT_WINDOW (user_data);
-
-	GtkClipboard *clip;
 	const gchar  *timestamp = NULL;
 
 	timestamp = pt_player_get_timestamp (win->priv->player);
-
-	if (timestamp) {
-		clip = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
-		gtk_clipboard_set_text (clip, timestamp, -1);
-	}
+	if (timestamp)
+		gtk_clipboard_set_text (win->priv->clip, timestamp, -1);
 }
 
 static void
@@ -88,12 +83,13 @@ clip_text_cb (GtkClipboard *clip,
 	      const gchar  *text,
 	      gpointer      data)
 {
-	PtPlayer *player = (PtPlayer *) data;
+	PtWindow *win = (PtWindow *) data;
 	gchar *timestamp;
 
 	if (text) {
 		timestamp = g_strdup (text);
-		pt_player_goto_timestamp (player, timestamp);
+		pt_player_goto_timestamp (win->priv->player, timestamp);
+		pt_waveviewer_set_follow_cursor (PT_WAVEVIEWER (win->priv->waveviewer), TRUE);
 		g_free (timestamp);
 	}
 }
@@ -106,10 +102,7 @@ insert_timestamp (GSimpleAction *action,
 	PtWindow *win;
 	win = PT_WINDOW (user_data);
 
-	GtkClipboard *clip;
-
-	clip = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
-	gtk_clipboard_request_text (clip, clip_text_cb, win->priv->player);
+	gtk_clipboard_request_text (win->priv->clip, clip_text_cb, win);
 }
 
 void
@@ -130,11 +123,23 @@ goto_position (GSimpleAction *action,
 	if (gtk_dialog_run (GTK_DIALOG (dlg)) == GTK_RESPONSE_OK) {
 		pos = pt_goto_dialog_get_pos (dlg);
 		pt_player_jump_to_position (win->priv->player, pos * 1000);
+		pt_waveviewer_set_follow_cursor (PT_WAVEVIEWER (win->priv->waveviewer), TRUE);
 	}
 
 	gtk_widget_destroy (GTK_WIDGET (dlg));
 }
 
+void
+goto_cursor (GSimpleAction *action,
+	     GVariant      *parameter,
+	     gpointer       user_data)
+{
+	PtWindow *win;
+	win = PT_WINDOW (user_data);
+
+	pt_waveviewer_set_follow_cursor (PT_WAVEVIEWER (win->priv->waveviewer), TRUE);
+}
+
 static void
 set_zoom (GSettings *editor,
           gint       step)
@@ -166,10 +171,172 @@ zoom_out (GSimpleAction *action,
 	set_zoom (win->priv->editor, -25);
 }
 
+static void
+setup_sphinx (PtWindow *win)
+{
+	GError *error = NULL;
+
+	pt_asr_settings_apply_config (
+			win->priv->asr_settings,
+			g_settings_get_string (win->priv->editor, "asr-config"),
+			win->priv->player);
+
+	pt_player_setup_sphinx (win->priv->player, &error);
+	if (error) {
+		pt_error_message (win, error->message, NULL);
+		g_clear_error (&error);
+	}
+}
+
+static void
+asr_output_search_cancelled_cb (PtAsrOutput *output,
+                                PtWindow    *win)
+{
+	GAction  *action;
+	GVariant *state;
+
+	g_signal_handler_disconnect (win->priv->output, win->priv->output_handler_id1);
+	g_signal_handler_disconnect (win->priv->output, win->priv->output_handler_id2);
+	g_clear_object (&win->priv->output);
+
+	action = g_action_map_lookup_action (G_ACTION_MAP (win), "mode");
+	state = g_variant_new_string ("playback");
+	g_simple_action_set_state (G_SIMPLE_ACTION (action), state);
+}
+
+static void
+asr_output_app_found_cb (PtAsrOutput *output,
+                         PtWindow    *win)
+{
+	GApplication  *app;
+	GNotification *notification;
+	gchar         *summary;
+	gchar         *message;
+
+	g_signal_handler_disconnect (win->priv->output, win->priv->output_handler_id1);
+	g_signal_handler_disconnect (win->priv->output, win->priv->output_handler_id2);
+
+	summary = g_strdup_printf (_("Found %s"), pt_asr_output_get_app_name (output));
+	message = _("Press “Play” to start automatic speech recognition.");
+	notification = g_notification_new (summary);
+	g_notification_set_body (notification, message);
+	app = G_APPLICATION (gtk_window_get_application (GTK_WINDOW (win)));
+	g_application_send_notification (app, "app-found", notification);
+
+	setup_sphinx (win);
+
+	g_application_withdraw_notification (app, "app-found");
+	g_object_unref (notification);
+	g_free (summary);
+}
+
+static void
+set_mode_asr (PtWindow *win)
+{
+	win->priv->output = pt_asr_output_new ();
+	win->priv->output_handler_id1 = g_signal_connect (
+			win->priv->output, "app-found",
+			G_CALLBACK (asr_output_app_found_cb), win);
+	win->priv->output_handler_id2 = g_signal_connect (
+			win->priv->output, "search-cancelled",
+			G_CALLBACK (asr_output_search_cancelled_cb), win);
+	pt_asr_output_search_app (win->priv->output, GTK_WINDOW (win));
+}
+
+static void
+asr_assistant_cancel_cb (GtkAssistant *assistant,
+                         gpointer      user_data)
+{
+	gtk_widget_destroy (GTK_WIDGET (assistant));
+}
+
+static void
+asr_assistant_close_cb (GtkAssistant *assistant,
+                        PtWindow     *win)
+{
+	gchar            *name;
+	gchar            *id;
+
+	name = pt_asr_assistant_get_name (PT_ASR_ASSISTANT (assistant));
+	id = pt_asr_assistant_save_config (
+			PT_ASR_ASSISTANT (assistant), win->priv->asr_settings);
+	g_settings_set_string (win->priv->editor, "asr-config", id);
+
+	gtk_widget_destroy (GTK_WIDGET (assistant));
+	g_free (name);
+	g_free (id);
+}
+
+static void
+launch_asr_assistant (PtWindow *win)
+{
+	GtkWidget *assistant;
+
+	assistant = GTK_WIDGET (pt_asr_assistant_new (GTK_WINDOW (win)));
+	g_signal_connect (assistant, "cancel", G_CALLBACK (asr_assistant_cancel_cb), NULL);
+	g_signal_connect (assistant, "close", G_CALLBACK (asr_assistant_close_cb), win);
+	gtk_widget_show (assistant);
+}
+
+static void
+set_mode_playback (PtWindow *win)
+{
+	GError *error = NULL;
+
+	pt_player_setup_player (win->priv->player, &error);
+	if (error) {
+		pt_error_message (win, error->message, NULL);
+		g_clear_error (&error);
+	}
+}
+
+void
+change_mode (GSimpleAction *action,
+             GVariant      *state,
+             gpointer       user_data)
+{
+	PtWindow *win = PT_WINDOW (user_data);
+	const gchar *mode;
+
+	if (!win->priv->asr_settings) {
+		GtkApplication *app;
+		app = gtk_window_get_application (GTK_WINDOW (win));
+		win->priv->asr_settings = g_object_ref (pt_app_get_asr_settings (PT_APP (app)));
+	}
+
+	mode = g_variant_get_string (state, NULL);
+	if (g_strcmp0 (mode, "playback") == 0) {
+		g_clear_object (&win->priv->output);
+		set_mode_playback (win);
+	} else if (g_strcmp0 (mode, "asr") == 0) {
+		if (pt_asr_settings_have_configs (win->priv->asr_settings)) {
+			/* TODO && have config saved in GSettings */
+			set_mode_asr (win);
+		} else {
+			launch_asr_assistant (win);
+			return;
+		}
+	} else {
+		g_assert_not_reached ();
+	}
+
+	g_simple_action_set_state (action, state);
+
+	/* On changing state the menu doesn’t close. It seems better to close
+	   it just like every other menu item closes the menu.
+	   Just in case this behaviour changes in the future, check if the
+	   menu is really open. */
+
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (win->priv->primary_menu_button)))
+		gtk_button_clicked (GTK_BUTTON (win->priv->primary_menu_button));
+}
+
 const GActionEntry win_actions[] = {
+	{ "mode", NULL, "s", "'playback'", change_mode },
 	{ "copy", copy_timestamp, NULL, NULL, NULL },
 	{ "insert", insert_timestamp, NULL, NULL, NULL },
 	{ "goto", goto_position, NULL, NULL, NULL },
+	{ "goto-cursor", goto_cursor, NULL, NULL, NULL },
 	{ "zoom-in", zoom_in, NULL, NULL, NULL },
 	{ "zoom-out", zoom_out, NULL, NULL, NULL }
 };
@@ -203,48 +370,6 @@ update_time_tick (GtkWidget     *widget,
 	return G_SOURCE_CONTINUE;
 }
 
-void
-cursor_changed_cb (GtkWidget *widget,
-		   gint64     pos,
-		   PtWindow  *win)
-{
-	/* triggered only by user */
-
-	pt_player_jump_to_position (win->priv->player, pos);
-	update_time (win);
-	pt_waveviewer_set_follow_cursor (PT_WAVEVIEWER (win->priv->waveviewer), TRUE);
-}
-
-void
-selection_changed_cb (GtkWidget *widget,
-		      PtWindow  *win)
-{
-	gint64 start, end, current;
-	if (win->priv->playing_selection) {
-
-		current = pt_player_get_position (win->priv->player);
-		g_object_get (win->priv->waveviewer,
-			      "selection-start", &start,
-			      "selection-end", &end,
-			      NULL);
-		if (start <= current && current <= end) {
-			pt_player_set_selection (win->priv->player, start, end);
-		} else {
-			pt_player_clear_selection (win->priv->player);
-			win->priv->playing_selection = FALSE;
-		}
-	}
-}
-
-void
-play_toggled_cb (GtkWidget *widget,
-		 PtWindow  *win)
-{
-	gtk_toggle_button_set_active (
-		GTK_TOGGLE_BUTTON (win->priv->button_play),
-		!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (win->priv->button_play)));
-}
-
 void
 zoom_in_cb (GtkWidget *widget,
 	    PtWindow  *win)
@@ -291,12 +416,14 @@ static void
 change_jump_back_tooltip (PtWindow *win)
 {
 	gchar *back;
+	gint   seconds;
 
+	seconds = pt_player_get_back (win->priv->player) / 1000;
 	back = g_strdup_printf (
 			ngettext ("Jump back 1 second",
 				  "Jump back %d seconds",
-				  win->priv->back),
-			win->priv->back);
+				  seconds),
+			seconds);
 
 	gtk_widget_set_tooltip_text (win->priv->button_jump_back, back);
 	g_free (back);
@@ -306,12 +433,14 @@ static void
 change_jump_forward_tooltip (PtWindow *win)
 {
 	gchar *forward;
+	gint   seconds;
 
+	seconds = pt_player_get_forward (win->priv->player) / 1000;
 	forward = g_strdup_printf (
 			ngettext ("Jump forward 1 second",
 				  "Jump forward %d seconds",
-				  win->priv->forward),
-			win->priv->forward);
+				  seconds),
+			seconds);
 
 	gtk_widget_set_tooltip_text (win->priv->button_jump_forward, forward);
 	g_free (forward);
@@ -321,17 +450,19 @@ static void
 change_play_button_tooltip (PtWindow *win)
 {
 	gchar    *play;
+	gint      pause;
 	gboolean  free_me = FALSE;
 
+	pause = pt_player_get_pause (win->priv->player) / 1000;
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (win->priv->button_play))) {
-		if (win->priv->pause == 0) {
+		if (pause == 0) {
 			play = _("Pause");
 		} else {
 			play = g_strdup_printf (
 					ngettext ("Pause and rewind 1 second",
 						  "Pause and rewind %d seconds",
-						  win->priv->pause),
-					win->priv->pause);
+						  pause),
+					pause);
 			free_me = TRUE;
 		}
 	} else {
@@ -350,16 +481,36 @@ update_insert_action_sensitivity_cb (GtkClipboard *clip,
 {
 	PtWindow *win = PT_WINDOW (data);
 	PtPlayer *player = win->priv->player;
-	gchar    *timestamp;
+	gchar    *timestamp = NULL;
 	gboolean  result = FALSE;
 	GAction  *action;
+	gint      pos;
+	gchar    *label;
 
 	if (text) {
 		timestamp = g_strdup (text);
-		result = pt_player_string_is_timestamp (player, timestamp);
+		pos = pt_player_get_timestamp_position (player, timestamp, TRUE);
 		g_free (timestamp);
+		timestamp = NULL;
+		if (pos >= 0) {
+			result = TRUE;
+			timestamp = pt_player_get_time_string (
+					pos,
+					pt_player_get_duration (player),
+					PT_PRECISION_SECOND_10TH);
+		}
 	}
 
+	if (timestamp)
+		label = g_strdup_printf (_("Go to time in clipboard: %s"), timestamp);
+	else
+		label = g_strdup (_("Go to time in clipboard"));
+
+	g_menu_item_set_label (win->priv->go_to_timestamp, label);
+	g_menu_remove (G_MENU (win->priv->secondary_menu), 1);
+	g_menu_insert_item (G_MENU (win->priv->secondary_menu), 1, win->priv->go_to_timestamp);
+	g_free (label);
+
 	action = g_action_map_lookup_action (G_ACTION_MAP (win), "insert");
 	g_simple_action_set_enabled (G_SIMPLE_ACTION (action), result);
 }
@@ -388,12 +539,27 @@ update_insert_action_sensitivity (GtkClipboard *clip,
 			win);
 }
 
+static void
+update_goto_cursor_action_sensitivity (PtWaveviewer *waveviewer,
+                                       gboolean      new,
+                                       gpointer      data)
+{
+	PtWindow  *win = PT_WINDOW (data);
+	GAction    *action;
+
+	action = g_action_map_lookup_action (G_ACTION_MAP (win), "goto-cursor");
+	g_simple_action_set_enabled (G_SIMPLE_ACTION (action), !new);
+}
+
 static void
 enable_win_actions (PtWindow *win,
 		    gboolean  state)
 {
-	GAction      *action;
-	GtkClipboard *clip;
+	GAction *action;
+
+	/* always active */
+	action = g_action_map_lookup_action (G_ACTION_MAP (win), "mode");
+	g_simple_action_set_enabled (G_SIMPLE_ACTION (action), TRUE);
 
 	action = g_action_map_lookup_action (G_ACTION_MAP (win), "copy");
 	g_simple_action_set_enabled (G_SIMPLE_ACTION (action), state);
@@ -403,13 +569,16 @@ enable_win_actions (PtWindow *win,
 		action = g_action_map_lookup_action (G_ACTION_MAP (win), "insert");
 		g_simple_action_set_enabled (G_SIMPLE_ACTION (action), state);
 	} else {
-		clip = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
-		update_insert_action_sensitivity (clip, NULL, win);
+		update_insert_action_sensitivity (win->priv->clip, NULL, win);
 	}
 
 	action = g_action_map_lookup_action (G_ACTION_MAP (win), "goto");
 	g_simple_action_set_enabled (G_SIMPLE_ACTION (action), state);
 
+	/* always insensitve: either there is no waveform or we are already at cursor position */
+	action = g_action_map_lookup_action (G_ACTION_MAP (win), "goto-cursor");
+	g_simple_action_set_enabled (G_SIMPLE_ACTION (action), FALSE);
+
 	action = g_action_map_lookup_action (G_ACTION_MAP (win), "zoom-in");
 	g_simple_action_set_enabled (G_SIMPLE_ACTION (action), state);
 
@@ -479,11 +648,10 @@ pt_window_ready_to_play (PtWindow *win,
 	   Reset tooltips for insensitive widgets. */
 
 	gchar     *display_name = NULL;
+	GtkStyleContext *open_context;
 
 	enable_win_actions (win, state);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (win->priv->button_play), FALSE);
-	gtk_widget_set_visible (win->priv->button_play, state);
-	gtk_widget_set_visible (win->priv->button_open, !state);
 
 	gtk_widget_set_sensitive (win->priv->button_play, state);
 	gtk_widget_set_sensitive (win->priv->button_fast_back, state);
@@ -493,6 +661,8 @@ pt_window_ready_to_play (PtWindow *win,
 	gtk_widget_set_sensitive (win->priv->speed_scale, state);
 
 	if (state) {
+		open_context = gtk_widget_get_style_context (win->priv->button_open);
+		gtk_style_context_remove_class (open_context, "suggested-action");
 		destroy_progress_dlg (win);
 		display_name = pt_player_get_filename (win->priv->player);
 		if (display_name) {
@@ -516,7 +686,7 @@ pt_window_ready_to_play (PtWindow *win,
 		pt_waveviewer_set_wave (PT_WAVEVIEWER (win->priv->waveviewer),
 					win->priv->wavedata);
 		pt_wavedata_free (win->priv->wavedata);
-		/* add timer after waveviewer, didn't update cursor otherwise sometimes */
+		/* add timer after waveviewer, didn’t update cursor otherwise sometimes */
 		add_timer (win);
 
 	} else {
@@ -529,18 +699,6 @@ pt_window_ready_to_play (PtWindow *win,
 	}
 }
 
-static void
-player_end_of_stream_cb (PtPlayer *player,
-			 PtWindow *win)
-{
-	/* Don't jump back */
-	gint temp;
-	temp = win->priv->pause;
-	win->priv->pause = 0;
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (win->priv->button_play), FALSE);
-	win->priv->pause = temp;
-}
-
 static void
 player_error_cb (PtPlayer *player,
 		 GError   *error,
@@ -548,7 +706,8 @@ player_error_cb (PtPlayer *player,
 {
 	destroy_progress_dlg (win);
 	pt_window_ready_to_play (win, FALSE);
-	pt_error_message (win, error->message);
+	pt_error_message (win, error->message, NULL);
+	/* TODO clear error? */
 }
 
 static void
@@ -562,7 +721,7 @@ open_cb (PtPlayer     *player,
 	destroy_progress_dlg (win);
 
 	if (!pt_player_open_uri_finish (player, res, &error)) {
-		pt_error_message (win, error->message);
+		pt_error_message (win, error->message, NULL);
 		g_error_free (error);
 		return;
 	}
@@ -570,6 +729,12 @@ open_cb (PtPlayer     *player,
 	pt_window_ready_to_play (win, TRUE);
 }
 
+gchar*
+pt_window_get_uri (PtWindow *win)
+{
+	return pt_player_get_uri (win->priv->player);
+}
+
 void
 pt_window_open_file (PtWindow *win,
 		     gchar    *uri)
@@ -582,64 +747,97 @@ pt_window_open_file (PtWindow *win,
 				  win);
 }
 
+static void
+update_play_after_toggle (PtWindow        *win,
+			  GtkToggleButton *button)
+{
+	if (gtk_toggle_button_get_active (button)) {
+		update_time (win);
+		pt_waveviewer_set_follow_cursor (PT_WAVEVIEWER (win->priv->waveviewer), TRUE);
+	}
+
+	change_play_button_tooltip (win);
+}
+
+
+static void
+player_play_toggled_cb (PtPlayer *player,
+			PtWindow *win)
+{
+	GtkToggleButton *play;
+	play = GTK_TOGGLE_BUTTON (win->priv->button_play);
+
+	/* Player changed play/pause; toggle GUI button and block signals */
+	g_signal_handlers_block_by_func (play, play_button_toggled_cb, win);
+	gtk_toggle_button_set_active (play, !gtk_toggle_button_get_active (play));
+	g_signal_handlers_unblock_by_func (play, play_button_toggled_cb, win);
+
+	update_play_after_toggle (win, play);
+}
+
 void
 play_button_toggled_cb (GtkToggleButton *button,
 			PtWindow	*win)
 {
-	gint64 start, end, current, dur;
-	gboolean selection;
+	/* GUI button toggled, block signals from PtPlayer */
+	g_signal_handlers_block_by_func (win->priv->player, player_play_toggled_cb, win);
+	pt_player_play_pause (win->priv->player);
+	g_signal_handlers_unblock_by_func (win->priv->player, player_play_toggled_cb, win);
 
-	if (gtk_toggle_button_get_active (button)) {
+	update_play_after_toggle (win, button);
+}
 
-		/* If there is a selection, play it */
-		g_object_get (win->priv->waveviewer,
-			      "selection-start", &start,
-			      "selection-end", &end,
-			      "has-selection", &selection,
-			      NULL);
-
-		if (!win->priv->playing_selection && selection) {
-			/* Note: changes position if outside selection */
-			pt_player_set_selection (win->priv->player, start, end);
-			win->priv->playing_selection = TRUE;
-		}
-
-		/* If we're at the end of stream or selection goto start */
-		current = pt_player_get_position (win->priv->player);
-		if (win->priv->playing_selection) {
-			if (current == end)
-				pt_player_jump_to_position (win->priv->player, start);
-		} else {
-			dur = pt_player_get_duration (win->priv->player);
-			/* We are usually not (never?) at the exact duration */
-			if (dur - current < 100)
-				pt_player_jump_to_position (win->priv->player, 0);
-		}
-
-		/* before following cursor update time, not to jump back and forth */
-		update_time (win);
-		pt_waveviewer_set_follow_cursor (PT_WAVEVIEWER (win->priv->waveviewer), TRUE);
-		pt_player_play (win->priv->player);
-	} else {
-		pt_player_pause (win->priv->player);
-		pt_player_jump_relative (win->priv->player, win->priv->pause * -1000);
-	}
+static void
+player_end_of_stream_cb (PtPlayer *player,
+			 PtWindow *win)
+{
+	/* Don’t jump back */
+	GtkToggleButton *play;
+	play = GTK_TOGGLE_BUTTON (win->priv->button_play);
 
+	g_signal_handlers_block_by_func (play, play_button_toggled_cb, win);
+	gtk_toggle_button_set_active (play, FALSE);
+	g_signal_handlers_unblock_by_func (play, play_button_toggled_cb, win);
+	pt_player_pause (win->priv->player);
 	change_play_button_tooltip (win);
 }
 
+static void
+player_jumped_back_cb (PtPlayer *player,
+                       PtWindow *win)
+{
+	GtkButton *button;
+	button = GTK_BUTTON (win->priv->button_jump_back);
+	g_signal_handlers_block_by_func (button, jump_back_button_clicked_cb, win);
+	gtk_button_clicked (button);
+	g_signal_handlers_unblock_by_func (button, jump_back_button_clicked_cb, win);
+}
+
 void
 jump_back_button_clicked_cb (GtkButton *button,
 			     PtWindow  *win)
 {
-	pt_player_jump_relative (win->priv->player, win->priv->back * -1000);
+	g_signal_handlers_block_by_func (win->priv->player, player_jumped_back_cb, win);
+	pt_player_jump_back (win->priv->player);
+	g_signal_handlers_unblock_by_func (win->priv->player, player_jumped_back_cb, win);
+}
+
+static void
+player_jumped_forward_cb (PtPlayer *player,
+                          PtWindow *win)
+{
+	GtkButton *button;
+	button = GTK_BUTTON (win->priv->button_jump_forward);
+	g_signal_handlers_block_by_func (button, jump_forward_button_clicked_cb, win);
+	gtk_button_clicked (button);
+	g_signal_handlers_unblock_by_func (button, jump_forward_button_clicked_cb, win);
 }
 
 void
 jump_forward_button_clicked_cb (GtkButton *button,
 			        PtWindow  *win)
 {
-	pt_player_jump_relative (win->priv->player, win->priv->forward * 1000);
+	pt_player_jump_forward (win->priv->player);
 }
 
 /* currently not used */
@@ -700,6 +898,23 @@ speed_changed_cb (GtkRange *range,
 	pt_player_set_speed (win->priv->player, win->priv->speed);
 }
 
+
+static void
+player_asr_final_cb (PtPlayer *player,
+                     gchar    *word,
+                     PtWindow *win)
+{
+	pt_asr_output_final (win->priv->output, word);
+}
+
+static void
+player_asr_hypothesis_cb (PtPlayer *player,
+                          gchar    *word,
+                          PtWindow *win)
+{
+	pt_asr_output_hypothesis (win->priv->output, word);
+}
+
 void
 speed_scale_direction_changed_cb (GtkWidget        *widget,
                                   GtkTextDirection  previous_direction,
@@ -805,24 +1020,67 @@ settings_changed_cb (GSettings *settings,
 	}
 }
 
+static gboolean
+map_seconds_to_milliseconds (GValue   *value,
+	                     GVariant *variant,
+	                     gpointer  data)
+{
+	/* Settings store seconds, PtPlayer wants milliseconds */
+	gint new;
+	new = g_variant_get_int32 (variant);
+	new = new * 1000;
+	g_value_set_int (value, new);
+	return TRUE;
+}
+
+static GVariant*
+map_milliseconds_to_seconds (const GValue       *value,
+	                     const GVariantType *type,
+	                     gpointer            data)
+{
+	gint new;
+	new = g_value_get_int (value);
+	new = new / 1000;
+	return g_variant_new_int32 (new);
+}
+
 static void
 setup_settings (PtWindow *win)
 {
 	win->priv->editor = g_settings_new ("com.github.gkarsay.parlatype");
 
-	g_settings_bind (
+	g_settings_bind_with_mapping (
 			win->priv->editor, "rewind-on-pause",
-			win, "pause",
-			G_SETTINGS_BIND_GET);
+			win->priv->player, "pause",
+			G_SETTINGS_BIND_GET,
+			map_seconds_to_milliseconds,
+			map_milliseconds_to_seconds,
+			NULL, NULL);
 
-	g_settings_bind (
+	g_settings_bind_with_mapping (
 			win->priv->editor, "jump-back",
-			win, "back",
+			win->priv->player, "back",
+			G_SETTINGS_BIND_GET,
+			map_seconds_to_milliseconds,
+			map_milliseconds_to_seconds,
+			NULL, NULL);
+
+	g_settings_bind_with_mapping (
+			win->priv->editor, "jump-forward",
+			win->priv->player, "forward",
+			G_SETTINGS_BIND_GET,
+			map_seconds_to_milliseconds,
+			map_milliseconds_to_seconds,
+			NULL, NULL);
+
+	g_settings_bind (
+			win->priv->editor, "repeat-all",
+			win->priv->player, "repeat-all",
 			G_SETTINGS_BIND_GET);
 
 	g_settings_bind (
-			win->priv->editor, "jump-forward",
-			win, "forward",
+			win->priv->editor, "repeat-selection",
+			win->priv->player, "repeat-selection",
 			G_SETTINGS_BIND_GET);
 
 	g_settings_bind (
@@ -835,6 +1093,26 @@ setup_settings (PtWindow *win)
 			win->priv->waveviewer, "fixed-cursor",
 			G_SETTINGS_BIND_GET);
 
+	g_settings_bind (
+			win->priv->editor, "timestamp-precision",
+			win->priv->player, "timestamp-precision",
+			G_SETTINGS_BIND_GET);
+
+	g_settings_bind (
+			win->priv->editor, "timestamp-fixed",
+			win->priv->player, "timestamp-fixed",
+			G_SETTINGS_BIND_GET);
+
+	g_settings_bind (
+			win->priv->editor, "timestamp-delimiter",
+			win->priv->player, "timestamp-delimiter",
+			G_SETTINGS_BIND_GET);
+
+	g_settings_bind (
+			win->priv->editor, "timestamp-fraction-sep",
+			win->priv->player, "timestamp-fraction-sep",
+			G_SETTINGS_BIND_GET);
+
 	/* connect to tooltip changer */
 
 	g_signal_connect (
@@ -876,8 +1154,23 @@ setup_settings (PtWindow *win)
 static void
 setup_player (PtWindow *win)
 {
-	/* Already tested in main.c, we don't check for errors here anymore */
-	win->priv->player = pt_player_new (NULL);
+	GError *error = NULL;
+
+	win->priv->player = pt_player_new ();
+	if (!pt_player_setup_player (win->priv->player, &error)) {
+		gchar *secondary_message = g_strdup_printf (
+			_("Parlatype needs GStreamer 1.x to run. Please check your installation of "
+                        "GStreamer and make sure you have the “Good Plugins” installed.\n"
+                        "Parlatype will quit now, it received this error message: %s"), error->message);
+		pt_error_message (win, _("Fatal error"), secondary_message);
+		g_clear_error (&error);
+		g_free (secondary_message);
+		exit (2);
+	}
+
+	pt_player_connect_waveviewer (
+			win->priv->player,
+			PT_WAVEVIEWER (win->priv->waveviewer));
 
 	g_signal_connect (win->priv->player,
 			"end-of-stream",
@@ -889,10 +1182,35 @@ setup_player (PtWindow *win)
 			G_CALLBACK (player_error_cb),
 			win);
 
+	g_signal_connect (win->priv->player,
+			"play-toggled",
+			G_CALLBACK (player_play_toggled_cb),
+			win);
+
+	g_signal_connect (win->priv->player,
+			"jumped-back",
+			G_CALLBACK (player_jumped_back_cb),
+			win);
+
+	g_signal_connect (win->priv->player,
+			"jumped-forward",
+			G_CALLBACK (player_jumped_forward_cb),
+			win);
+
 	g_object_bind_property (
 			win->priv->volumebutton, "value",
 			win->priv->player, "volume",
 			G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE);
+
+	g_signal_connect (win->priv->player,
+			"asr-final",
+			G_CALLBACK (player_asr_final_cb),
+			win);
+
+	g_signal_connect (win->priv->player,
+			"asr-hypothesis",
+			G_CALLBACK (player_asr_hypothesis_cb),
+			win);
 }
 
 static void
@@ -908,39 +1226,38 @@ setup_accels_actions_headerbar (PtWindow *win)
 
 	/* GtkHeader workaround for glade 3.16 + Menu button */
 	GtkBuilder    *builder;
-	GMenuModel    *model;
 	GtkWidget     *hbar;
-	GtkWidget     *menu_button;
+	GMenuModel    *primary_menu;
 
 	builder = gtk_builder_new_from_resource ("/com/github/gkarsay/parlatype/window-headerbar.ui");
 	hbar = GTK_WIDGET (gtk_builder_get_object (builder, "headerbar"));
 	win->priv->button_open = GTK_WIDGET (gtk_builder_get_object (builder, "button_open"));
-	win->priv->button_play = GTK_WIDGET (gtk_builder_get_object (builder, "button_play"));
-	menu_button = GTK_WIDGET (gtk_builder_get_object (builder, "menu_button"));
-
-#if GTK_CHECK_VERSION(3,14,0)
-#else
-	/* In GTK+ 3.10 the new icon doesn't exist, don't know 3.12, works in 3.14 */
-	GtkWidget *image;
-	image = gtk_image_new_from_icon_name ("emblem-system-symbolic", GTK_ICON_SIZE_MENU);
-	gtk_button_set_image (GTK_BUTTON (menu_button), image);
-#endif
-
+	win->priv->primary_menu_button = GTK_WIDGET (gtk_builder_get_object (builder, "primary_menu_button"));
 	gtk_window_set_titlebar (GTK_WINDOW (win), hbar);
 	gtk_builder_connect_signals (builder, win);
 	g_object_unref (builder);
 
 	builder = gtk_builder_new_from_resource ("/com/github/gkarsay/parlatype/menus.ui");
-	model = G_MENU_MODEL (gtk_builder_get_object (builder, "winmenu"));
-	gtk_menu_button_set_menu_model (GTK_MENU_BUTTON (menu_button), model);
+	primary_menu = G_MENU_MODEL (gtk_builder_get_object (builder, "primary-menu"));
+	win->priv->secondary_menu = G_MENU_MODEL (gtk_builder_get_object (builder, "secondary-menu"));
+	gtk_menu_button_set_menu_model (GTK_MENU_BUTTON (win->priv->primary_menu_button), primary_menu);
+	gtk_menu_button_set_menu_model (GTK_MENU_BUTTON (win->priv->pos_menu_button), win->priv->secondary_menu);
 	g_object_unref (builder);
+	win->priv->go_to_timestamp = g_menu_item_new (_("Go to time in clipboard"), "win.insert");
+	g_menu_insert_item (G_MENU (win->priv->secondary_menu), 1, win->priv->go_to_timestamp);
+
+	GApplication *app;
+	app = g_application_get_default ();
+	if (!pt_app_get_asr (PT_APP (app))) {
+		g_menu_remove (G_MENU (primary_menu), 0);
+	}
 
 	/* Accels */
 	win->priv->accels = gtk_accel_group_new ();
 	gtk_window_add_accel_group (GTK_WINDOW (win), win->priv->accels);
 	
 	gtk_widget_add_accelerator (
-			menu_button,
+			win->priv->primary_menu_button,
 			"clicked",
 			win->priv->accels,
 			GDK_KEY_F10,
@@ -981,50 +1298,30 @@ setup_accels_actions_headerbar (PtWindow *win)
 			GTK_ACCEL_VISIBLE);
 }
 
-void
-goto_toggled_cb (GtkToggleButton *button,
-		 gpointer         data)
-{
-	gchar *tooltip;
-
-	if (gtk_toggle_button_get_active (button))
-		tooltip = _("View follows cursor");
-	else
-		tooltip = _("Click to follow cursor");
-
-	gtk_widget_set_tooltip_text (GTK_WIDGET (button), tooltip);
-}
-
-static void
-setup_goto_button (PtWindow *win)
-{
-	g_object_bind_property (
-			win->priv->waveviewer, "follow-cursor",
-			win->priv->goto_button, "active",
-			G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE);
-}
-
 static void
 pt_window_init (PtWindow *win)
 {
 	win->priv = pt_window_get_instance_private (win);
-	GtkClipboard *clip;
 
 	gtk_widget_init_template (GTK_WIDGET (win));
 
-	setup_settings (win);
 	setup_player (win);
+	setup_settings (win);
 	setup_accels_actions_headerbar (win);
-	setup_goto_button (win);
 	setup_mediakeys (win);		/* this is in pt_mediakeys.c */
 	pt_window_setup_dnd (win);	/* this is in pt_window_dnd.c */
 	setup_dbus_service (win);	/* this is in pt_dbus_service.c */
+	win->priv->asr_settings = NULL;
+	win->priv->output = NULL;
+	win->priv->output_handler_id1 = 0;
+	win->priv->output_handler_id2 = 0;
 	win->priv->recent = gtk_recent_manager_get_default ();
 	win->priv->timer = 0;
 	win->priv->progress_dlg = NULL;
 	win->priv->progress_handler_id = 0;
 	win->priv->wavedata = NULL;
-	win->priv->playing_selection = FALSE;
+	win->priv->clip_handler_id = 0;
+	win->priv->clip = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
 
 	/* Used e.g. by Xfce */
 	gtk_window_set_default_icon_name ("com.github.gkarsay.parlatype");
@@ -1035,11 +1332,14 @@ pt_window_init (PtWindow *win)
 
 	pt_window_ready_to_play (win, FALSE);
 
-	clip = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
-	g_signal_connect (clip,
+	win->priv->clip_handler_id = g_signal_connect (win->priv->clip,
 			"owner-change",
 			G_CALLBACK (update_insert_action_sensitivity),
 			win);
+	g_signal_connect (win->priv->waveviewer,
+			"follow-cursor-changed",
+			G_CALLBACK (update_goto_cursor_action_sensitivity),
+			win);
 }
 
 static void
@@ -1062,120 +1362,39 @@ pt_window_dispose (GObject *object)
 		g_settings_set_int (win->priv->editor, "height", y);
 	}
 
+	if (win->priv->clip_handler_id > 0) {
+		g_signal_handler_disconnect (win->priv->clip, win->priv->clip_handler_id);
+		win->priv->clip_handler_id = 0;
+	}
 	remove_timer (win);
 	g_clear_object (&win->priv->editor);
 	g_clear_object (&win->priv->proxy);
 	destroy_progress_dlg (win);
 	g_clear_object (&win->priv->player);
+	g_clear_object (&win->priv->output);
+	g_clear_object (&win->priv->asr_settings);
 
 	G_OBJECT_CLASS (pt_window_parent_class)->dispose (object);
 }
 
-static void
-pt_window_set_property (GObject      *object,
-			guint         property_id,
-			const GValue *value,
-			GParamSpec   *pspec)
-{
-	PtWindow *win;
-	win = PT_WINDOW (object);
-
-	switch (property_id) {
-	case PROP_PAUSE:
-		win->priv->pause = g_value_get_int (value);
-		break;
-	case PROP_BACK:
-		win->priv->back = g_value_get_int (value);
-		break;
-	case PROP_FORWARD:
-		win->priv->forward = g_value_get_int (value);
-		break;
-	default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
-		break;
-	}
-}
-
-static void
-pt_window_get_property (GObject    *object,
-			guint       property_id,
-			GValue     *value,
-			GParamSpec *pspec)
-{
-	PtWindow *win;
-	win = PT_WINDOW (object);
-
-	switch (property_id) {
-	case PROP_PAUSE:
-		g_value_set_int (value, win->priv->pause);
-		break;
-	case PROP_BACK:
-		g_value_set_int (value, win->priv->back);
-		break;
-	case PROP_FORWARD:
-		g_value_set_int (value, win->priv->forward);
-		break;
-	default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
-		break;
-	}
-}
-
 static void
 pt_window_class_init (PtWindowClass *klass)
 {
 	GObjectClass   *gobject_class = G_OBJECT_CLASS (klass);
 	GtkWidgetClass *widget_class  = GTK_WIDGET_CLASS (klass);
 
-	gobject_class->set_property = pt_window_set_property;
-	gobject_class->get_property = pt_window_get_property;
 	gobject_class->dispose      = pt_window_dispose;
 	gtk_widget_class_set_template_from_resource (widget_class, "/com/github/gkarsay/parlatype/window.ui");
+	gtk_widget_class_bind_template_child_private (widget_class, PtWindow, button_play);
 	gtk_widget_class_bind_template_child_private (widget_class, PtWindow, button_fast_back);	// not used
 	gtk_widget_class_bind_template_child_private (widget_class, PtWindow, button_fast_forward);	// not used
 	gtk_widget_class_bind_template_child_private (widget_class, PtWindow, button_jump_back);
 	gtk_widget_class_bind_template_child_private (widget_class, PtWindow, button_jump_forward);
 	gtk_widget_class_bind_template_child_private (widget_class, PtWindow, volumebutton);
+	gtk_widget_class_bind_template_child_private (widget_class, PtWindow, pos_menu_button);
 	gtk_widget_class_bind_template_child_private (widget_class, PtWindow, pos_label);
-	gtk_widget_class_bind_template_child_private (widget_class, PtWindow, label_box);
-	gtk_widget_class_bind_template_child_private (widget_class, PtWindow, goto_button);
 	gtk_widget_class_bind_template_child_private (widget_class, PtWindow, speed_scale);
 	gtk_widget_class_bind_template_child_private (widget_class, PtWindow, waveviewer);
-
-	obj_properties[PROP_PAUSE] =
-	g_param_spec_int (
-			"pause",
-			"Seconds to rewind on pause",
-			"Seconds to rewind on pause",
-			0,	/* minimum */
-			10,	/* maximum */
-			0,
-			G_PARAM_READWRITE);
-
-	obj_properties[PROP_BACK] =
-	g_param_spec_int (
-			"back",
-			"Seconds to jump back",
-			"Seconds to jump back",
-			1,	/* minimum */
-			60,	/* maximum */
-			10,
-			G_PARAM_READWRITE);
-
-	obj_properties[PROP_FORWARD] =
-	g_param_spec_int (
-			"forward",
-			"Seconds to jump forward",
-			"Seconds to jump forward",
-			1,	/* minimum */
-			60,	/* maximum */
-			10,
-			G_PARAM_READWRITE);
-
-	g_object_class_install_properties (
-			gobject_class,
-			N_PROPERTIES,
-			obj_properties);
 }
 
 PtWindow *
